import { ReadableStream, Response } from './w3-mocks.mjs';
export const DEFAULT_TRANSFORM_RESULT_FN = (result) => result;
export function getRegularResponse(executionResult, transformResult = DEFAULT_TRANSFORM_RESULT_FN) {
    const headersInit = [];
    const responseInit = {
        headers: headersInit,
        status: 200,
    };
    const transformedResult = transformResult(executionResult);
    const responseBody = JSON.stringify(transformedResult);
    return new Response(responseBody, responseInit);
}
export function getMultipartResponse(asyncExecutionResult, transformResult = DEFAULT_TRANSFORM_RESULT_FN) {
    const headersInit = {
        Connection: "keep-alive",
        "Content-Type": 'multipart/mixed; boundary="-"',
        "Transfer-Encoding": "chunked",
    };
    const responseInit = {
        headers: headersInit,
        status: 200,
    };
    const readableStream = new ReadableStream({
        async start(controller) {
            try {
                controller.enqueue(`---`);
                const iterator = asyncExecutionResult[Symbol.asyncIterator]();
                while (true) {
                    const { done, value } = await iterator.next();
                    if (done) {
                        controller.enqueue("\r\n-----\r\n");
                        controller.close();
                        break;
                    }
                    const transformedResult = transformResult(value);
                    const chunk = JSON.stringify(transformedResult);
                    const data = [
                        "",
                        "Content-Type: application/json; charset=utf-8",
                        "Content-Length: " + String(chunk.length),
                        "",
                        chunk,
                    ];
                    if (value.hasNext) {
                        data.push("---");
                    }
                    controller.enqueue(data.join("\r\n"));
                }
            }
            catch (e) {
                controller.error(e);
            }
        },
    });
    return new Response(readableStream, responseInit);
}
export function getPushResponse(asyncExecutionResult, transformResult = DEFAULT_TRANSFORM_RESULT_FN) {
    const headersInit = {
        "Content-Type": "text/event-stream",
        Connection: "keep-alive",
        "Cache-Control": "no-cache",
    };
    const responseInit = {
        headers: headersInit,
        status: 200,
    };
    const readableStream = new ReadableStream({
        async start(controller) {
            try {
                const iterator = asyncExecutionResult[Symbol.asyncIterator]();
                while (true) {
                    const { done, value } = await iterator.next();
                    if (done) {
                        controller.close();
                        break;
                    }
                    const transformedResult = transformResult(value);
                    const chunk = JSON.stringify(transformedResult);
                    controller.enqueue(`data: ${chunk}\n\n`);
                }
                controller.close();
            }
            catch (e) {
                controller.error(e);
            }
        },
    });
    return new Response(readableStream, responseInit);
}
async function* getSingleResult(payload) {
    yield payload;
}
export function getErrorResponse({ message, status = 500, headers = {}, errors = [{ message }], transformResult = DEFAULT_TRANSFORM_RESULT_FN, isEventStream, }) {
    const payload = {
        errors,
    };
    if (isEventStream) {
        return getPushResponse(getSingleResult(payload), transformResult);
    }
    return new Response(JSON.stringify(transformResult(payload)), {
        status,
        headers,
    });
}
