import { isAsyncIterable } from './is-async-iterable.mjs';
import { Request, ReadableStream } from './w3-mocks.mjs';
export async function getNodeRequest(nodeRequest) {
    const fullUrl = `${nodeRequest.protocol || "http"}://${nodeRequest.hostname || nodeRequest.headers.host || "localhost"}${nodeRequest.url || '/graphql'}`;
    if (nodeRequest.method !== "POST") {
        return new Request(fullUrl, {
            headers: nodeRequest.headers,
            method: nodeRequest.method,
        });
    }
    else if (nodeRequest.body) {
        return new Request(fullUrl, {
            headers: nodeRequest.headers,
            method: nodeRequest.method,
            body: JSON.stringify(nodeRequest.body),
        });
    }
    else if (isAsyncIterable(nodeRequest)) {
        const body = new ReadableStream({
            async start(controller) {
                for await (const chunk of nodeRequest) {
                    controller.enqueue(chunk);
                }
                controller.close();
            },
        });
        return new Request(fullUrl, {
            headers: nodeRequest.headers,
            method: nodeRequest.method,
            body,
        });
    }
    throw new Error(`Unknown request`);
}
export async function sendNodeResponse(responseResult, nodeResponse) {
    const headersObj = {};
    responseResult.headers.forEach((value, name) => {
        headersObj[name] = headersObj[name] || [];
        headersObj[name].push(value);
    });
    nodeResponse.writeHead(responseResult.status, headersObj);
    const responseBody = await responseResult.body;
    if (responseBody == null) {
        throw new Error("Response body is not supported");
    }
    if (responseBody instanceof Uint8Array) {
        nodeResponse.write(responseBody);
        nodeResponse.end();
    }
    else if (isAsyncIterable(responseBody)) {
        for await (const chunk of responseBody) {
            if (chunk) {
                nodeResponse.write(chunk);
            }
        }
        nodeResponse.end();
    }
    else if ("getReader" in responseBody) {
        const reader = responseBody.getReader();
        while (true) {
            const { done, value } = await reader.read();
            if (value) {
                nodeResponse.write(value);
            }
            if (done) {
                nodeResponse.end();
                break;
            }
        }
        nodeResponse.on("close", () => {
            reader.releaseLock();
        });
    }
    else {
        throw new Error(`Unrecognized Response type provided`);
    }
}
