import { isAsyncIterable } from './is-async-iterable.mjs';
export const ReadableStream = globalThis.ReadableStream || class ReadableStreamPonyfill {
    constructor(source) {
        this.source = source;
    }
    getReader() {
        var _a, _b;
        let listening = true;
        const pullQueue = [];
        const pushQueue = [];
        const pushValue = (value) => {
            if (pullQueue.length !== 0) {
                pullQueue.shift()({ value, done: false });
            }
            else {
                pushQueue.push({ value, done: false });
            }
        };
        const pushDone = () => {
            if (pullQueue.length !== 0) {
                pullQueue.shift()({ value: undefined, done: true });
            }
            else {
                pushQueue.push({ value: undefined, done: true });
            }
        };
        const pullValue = () => new Promise(resolve => {
            if (pushQueue.length !== 0) {
                const element = pushQueue.shift();
                resolve(element);
            }
            else {
                pullQueue.push(resolve);
            }
        });
        const emptyQueue = () => {
            if (listening) {
                listening = false;
                for (const resolve of pullQueue) {
                    resolve({ value: undefined, done: true });
                }
                pullQueue.length = 0;
                pushQueue.length = 0;
            }
            return Promise.resolve({ value: undefined, done: true });
        };
        const controller = {
            desiredSize: Infinity,
            error: () => {
            },
            enqueue: (value) => {
                pushValue(value);
            },
            close: () => {
                pushDone();
            }
        };
        (_b = (_a = this.source).start) === null || _b === void 0 ? void 0 : _b.call(_a, controller);
        return {
            async read() {
                return listening ? pullValue() : emptyQueue();
            },
            releaseLock: () => {
                var _a, _b;
                emptyQueue();
                (_b = (_a = this.source).cancel) === null || _b === void 0 ? void 0 : _b.call(_a, controller);
            },
            cancel: async () => {
                var _a, _b;
                emptyQueue();
                (_b = (_a = this.source).cancel) === null || _b === void 0 ? void 0 : _b.call(_a, controller);
            },
            closed: Promise.resolve(undefined),
        };
    }
};
class BodyPonyfill {
    constructor(source) {
        this.source = source;
    }
    async text() {
        if (this.source != null) {
            if (typeof this.source === "string") {
                return this.source;
            }
            else if (isAsyncIterable(this.source)) {
                const chunks = [];
                for await (const chunk of this.source) {
                    chunks.push(chunk);
                }
                return chunks.join("");
            }
            else if ("getReader" in this.source) {
                const reader = this.source.getReader();
                const chunks = [];
                while (true) {
                    const { done, value } = await reader.read();
                    if (done) {
                        break;
                    }
                    chunks.push(value);
                }
                return chunks.join("");
            }
        }
        return undefined;
    }
    async json() {
        const text = await this.text();
        if (text) {
            return JSON.parse(text);
        }
    }
    get body() {
        const oneTimeGenerator = (val) => async function* () {
            yield val;
        };
        if (typeof this.source === "string") {
            return {
                getReader: () => ({
                    read: async () => ({
                        done: true,
                        value: this.source,
                    }),
                    releaseLock: () => { },
                }),
                [Symbol.asyncIterator]: oneTimeGenerator(this.source),
            };
        }
        else {
            return this.source;
        }
    }
}
export const Request = globalThis.Request ||
    class Request extends BodyPonyfill {
        constructor(url, init) {
            super(init === null || init === void 0 ? void 0 : init.body);
            this.url = url;
            this.init = init;
        }
        get headers() {
            return {
                get: (name) => {
                    var _a, _b;
                    return (_b = (_a = this.init) === null || _a === void 0 ? void 0 : _a.headers) === null || _b === void 0 ? void 0 : _b[name];
                },
                forEach: (fn) => {
                    var _a;
                    const headersObj = ((_a = this.init) === null || _a === void 0 ? void 0 : _a.headers) || {};
                    for (const key in headersObj) {
                        fn(headersObj[key], key);
                    }
                },
            };
        }
        get method() {
            var _a, _b;
            return (_b = (_a = this.init) === null || _a === void 0 ? void 0 : _a.method) !== null && _b !== void 0 ? _b : "GET";
        }
    };
export const Response = globalThis.Response ||
    class Response extends BodyPonyfill {
        constructor(source, init) {
            super(source);
            this.init = init;
        }
        get headers() {
            return {
                get: (name) => {
                    var _a, _b;
                    return (_b = (_a = this.init) === null || _a === void 0 ? void 0 : _a.headers) === null || _b === void 0 ? void 0 : _b[name];
                },
                forEach: (fn) => {
                    var _a;
                    const headersObj = ((_a = this.init) === null || _a === void 0 ? void 0 : _a.headers) || {};
                    for (const key in headersObj) {
                        fn(headersObj[key], key);
                    }
                },
            };
        }
        get status() {
            var _a, _b;
            return (_b = (_a = this.init) === null || _a === void 0 ? void 0 : _a.status) !== null && _b !== void 0 ? _b : 200;
        }
    };
