"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendNodeResponse = exports.getNodeRequest = void 0;
const is_async_iterable_1 = require("./is-async-iterable");
const w3_mocks_1 = require("./w3-mocks");
async function getNodeRequest(nodeRequest) {
    const fullUrl = `${nodeRequest.protocol || "http"}://${nodeRequest.hostname || nodeRequest.headers.host || "localhost"}${nodeRequest.url || '/graphql'}`;
    if (nodeRequest.method !== "POST") {
        return new w3_mocks_1.Request(fullUrl, {
            headers: nodeRequest.headers,
            method: nodeRequest.method,
        });
    }
    else if (nodeRequest.body) {
        return new w3_mocks_1.Request(fullUrl, {
            headers: nodeRequest.headers,
            method: nodeRequest.method,
            body: JSON.stringify(nodeRequest.body),
        });
    }
    else if ((0, is_async_iterable_1.isAsyncIterable)(nodeRequest)) {
        const body = new w3_mocks_1.ReadableStream({
            async start(controller) {
                for await (const chunk of nodeRequest) {
                    controller.enqueue(chunk);
                }
                controller.close();
            },
        });
        return new w3_mocks_1.Request(fullUrl, {
            headers: nodeRequest.headers,
            method: nodeRequest.method,
            body,
        });
    }
    throw new Error(`Unknown request`);
}
exports.getNodeRequest = getNodeRequest;
async function sendNodeResponse(responseResult, nodeResponse) {
    const headersObj = {};
    responseResult.headers.forEach((value, name) => {
        headersObj[name] = headersObj[name] || [];
        headersObj[name].push(value);
    });
    nodeResponse.writeHead(responseResult.status, headersObj);
    const responseBody = await responseResult.body;
    if (responseBody == null) {
        throw new Error("Response body is not supported");
    }
    if (responseBody instanceof Uint8Array) {
        nodeResponse.write(responseBody);
        nodeResponse.end();
    }
    else if ((0, is_async_iterable_1.isAsyncIterable)(responseBody)) {
        for await (const chunk of responseBody) {
            if (chunk) {
                nodeResponse.write(chunk);
            }
        }
        nodeResponse.end();
    }
    else if ("getReader" in responseBody) {
        const reader = responseBody.getReader();
        while (true) {
            const { done, value } = await reader.read();
            if (value) {
                nodeResponse.write(value);
            }
            if (done) {
                nodeResponse.end();
                break;
            }
        }
        nodeResponse.on("close", () => {
            reader.releaseLock();
        });
    }
    else {
        throw new Error(`Unrecognized Response type provided`);
    }
}
exports.sendNodeResponse = sendNodeResponse;
